require("util")
require("homeworld_defines")
homeworld_config = require('config')
require("helpers.assertion")
require("helpers.actor")
require("helpers.misc_helpers")
require("helpers.gui_helpers")
require("homeworld_logic")
require("actors.sawmill")
require("actors.farm")
require("actors.fishery")
require("actors.seeder")
require("actors.terraformer")
require("actors.portal")
require("actors.belt_throughput_reader")
require("actors.belt_gate")
require("actors.homeworld_speaker")

register_entity_actor(Sawmill, "sawmill")
register_entity_actor(Farm, "farm")
register_entity_actor(Fishery, "fishery")
register_entity_actor(Seeder, "seeder")
register_entity_actor(Terraformer, "terraformer")
register_entity_actor(Portal, "homeworld_portal")
register_entity_actor(BeltThroughputReader, "belt_throughput_reader")
register_entity_actor(BeltGate, "belt_gate")
register_entity_actor(HomeworldSpeaker, "homeworld-speaker")

function on_mod_init( event )
    GUI.init()
    Homeworld:init()
    --If mod was loaded into a save, give a portal to all players
    for _, player in pairs(game.players) do
        give_items(player)
        --Also load up the new GUI
        if not player.gui.top["Homeworld_btn"] then
            player.gui.top.add{type="button",caption="HW:R",name="Homeworld_btn"}
        end
    end
end

function on_mod_load( event )
    Homeworld:load()
    load_persisted_actors()
end

function on_built_entity( event )
    --Tracks registered Actor entities
    event_create_entity(event.created_entity)
end

function on_entity_died( event )
    --Tracks registered Actor entities
    event_destroy_entity(event.entity)
end

function on_tick( event )
    --Ticks entities we've registered as an Actor
    tick_all_actors(event.tick)
    --Tick the homeworld control
    Homeworld:tick(event.tick)
    if ModuloTimer(20) then
    	Fishery.update_fish_chunks()
    end
end

function give_items( player)
    --Give homeworld portal to player
    player.insert{
        name = "homeworld_portal",
        count = 1
   }
end

function on_player_created( event )
    --New players get a homeworld portal
    local player = game.players[event.player_index]
    give_items(player)
    
    --Add HW GUI button
    if not player.gui.top["Homeworld_btn"] then
        player.gui.top.add{type="button",caption="HW:R",name="Homeworld_btn"}
    end
end

function on_config_change( event )
    --Ensure the mod GUI is on for all players
    for _, player in pairs(game.players) do
        --Also load up the new GUI
        --PrintToAllPlayers(serpent.block(player.gui.top["Homeworld_btn"]))
        --top[creative_mode_defines.names.gui.main_menu_open_button]
        if not player.gui.top["Homeworld_btn"] then
            player.gui.top.add{type="button",caption="HW:R",name="Homeworld_btn"}
        end
    end
end
script.on_configuration_changed(on_config_change)

function on_resource_depleted( event )
    --Depleted sand resource turns to dry-dirt tile
	if event.entity.name == "sand-source" then
		event.entity.surface.set_tiles{{name = "dry-dirt", position = event.entity.position}}
	end
end

function on_player_respawned(event)
    --Player was respawned, re-add GUI.
    local player = game.players[event.player_index]
    --Add HW GUI button
    if not player.gui.top["Homeworld_btn"] then
        player.gui.top.add{type="button",caption="HW:R",name="Homeworld_btn"}
    end
end

function toggle_homeworld_gui(event)
    if not (event.element.name == "Homeworld_btn") then
        return
    end
    if Homeworld.state.gui[event.player_index] then
        Homeworld:hide_gui(event.player_index)
    else
        Homeworld:show_gui(event.player_index)
    end
end

function before_player_mined_item( event )
    --Tracks registered Actor entities
    event_destroy_entity(event.entity)
end

function before_robot_mined( event )
    --Tracks registered Actor entities
    event_destroy_entity(event.entity)
end

function event_create_entity( entity )
    -- TODO(luke): move this out of control.lua, obviously the best place for this would be in farm.lua
    -- and that will require some changes to the actor system.
	if entity.name == 'entity-ghost' and entity.ghost_prototype and entity.ghost_prototype.name then
		local ghost_name = entity.ghost_prototype.name
		
		if ghost_name == "farm_01" or ghost_name == "farm_02" or ghost_name == "farm_03" or ghost_name == "farm_full" then
			--Get position of entity
			local position = entity.position
			local surface_name = entity.surface.name
			local name = entity.name
			
			--Destroy old farm
			entity.destroy()
			
			--Create proper entity
			local new_farm = game.surfaces['nauvis'].create_entity{
               name = "entity-ghost",
               position = position,
               force = game.forces.player,
			   inner_name = "stone-furnace"
            }
			
			create_entity_actor(new_farm)
			return
		end
	end
	create_entity_actor(entity)
end

function event_destroy_entity( entity )
    --Tracks registered Actor entities
    --Don't think this is used at all anymore.
    destroy_entity_actor(entity)
end

script.on_init(on_mod_init)
script.on_load(on_mod_load)
script.on_event(defines.events.on_built_entity, on_built_entity)
script.on_event(defines.events.on_robot_built_entity, on_built_entity)
script.on_event(defines.events.on_entity_died, on_entity_died)
script.on_event(defines.events.on_player_created, on_player_created)
script.on_event(defines.events.on_player_joined_game, on_player_load)
script.on_event(defines.events.on_pre_player_mined_item, before_player_mined_item)
script.on_event(defines.events.on_robot_pre_mined, before_robot_mined)
script.on_event(defines.events.on_resource_depleted, on_resource_depleted)
script.on_event(defines.events.on_tick, on_tick)
script.on_event(defines.events.on_player_respawned, on_player_respawned)
script.on_event(defines.events.on_gui_click,toggle_homeworld_gui)

--Remote interface for letting items call into Homeworld script
remote.add_interface("homeworld", {
	get_homeworld = function()
      return Homeworld
	end,

   show_gui = function()
      Homeworld:show_gui(game.local_player.index)
   end,

   hide_gui = function( player_index )
      Homeworld:hide_gui(game.local_player.index)
   end,

   add_item = function( name, count )
      Homeworld:insert_item{name = name, count = count}
   end,

   remove_item = function( name, count )
      Homeworld:remove_item{name = name, count = count}
   end,
})
